import numpy as np
from pyne.material import Material
from pyne.mcnp import mats_from_inp
from pyne.mesh import Mesh, IMeshTag

from gtcadis_tools import eta_sweep, plot_eta_sweep

# These materials (aside from vanadium alloy) are from the ARIES material library, produced by Dr. Laila El-Guebaly, 
# elguebaly@engr.wisc.edu, at the University of Wisconsin - Madison.
mats = [
   # Vanadium alloy
   # PROPERTIES OF V-4CR-4TI FOR APPLICATION AS FUSION REACTOR STRUCTURAL COMPONENTS
   # H. M. Chung, B. A. Loomis, and D. L. Smith, Argonne National Laboratory, August 1994
    Material(
        {'v': 43.62,
        'cr': 5.74,
        'ti': 10.64,
        'o': 350e-4,
        'n': 220e-4, 
        'c': 200e-4, 
        'si': 870e-4, 
        's': 20e-4, 
        'p': 40e-4, 
        'nb': 100e-4, 
        'mo': 100e-4}, density =6.05, metadata = {"name":"V-4Cr-4Ti"}),
    # ARIES-AT 90% Enriched Lithium Lead at 580 C
    # S. Malang & K. Schleisiek, "Dual Coolant Blanket Concept", KFK5424, Karlsruhe, Nov 1994
    Material(
         {'pb': 99.29,
         'li6': 0.62,  
         'li7': 0.08, 
         'zn': 10e-4,
         'fe': 10e-4,
         'bi': 43e-4,
         'cd': 5e-4, 
         'ag': 5e-4, 
         'sn': 5e-4, 
         'ni': 2e-4}, density=9, metadata={"name":"li17pb83_90enc"}),
    # Inconel-625
    # ARIES-AT magnet, P. Heitzenroeder - 6/30/2000
    Material({'c':0.1, 'al':0.4, 'si':0.5, 'p':0.15, 's':0.015, 'ti':0.4, 'cr':21.5, 'mn':0.5, 
              'fe':5, 'co':1, 'ni':58, 'nb':1.72, 'mo':9, 'ta':1.72}, density=8.44, metadata={"name":"Inconel"}),
    # ARIES-AT Silicon-Carbide
    # S. Sharaft, IPFR/UCLA, Aries Study: Materials: Sept. 1993, pg 3
    # modifications relative to reference, 04/28/2000 PPHW (initial definition)
    Material( {'c': 29.95,   'na': 0.050e-4, 'si': 70.05,   'k': 0.180e-4,
              'sc': 0.013e-4, 'cr': 0.017e-4, 'fe': 0.440e-4, 'co': 0.013e-4, 'ni': 0.074e-4,
              'cu': 0.048e-4, 'zn': 0.043e-4, 'ga': 0.005e-4, 'as': 0.003e-4, 'se': 0.001e-4,
              'br': 0.001e-4, 'rb': 0.001e-4, 'sr': 0.012e-4, 'zr': 0.236e-4, 'mo': 0.041e-4,
              'ag': 0.002e-4, 'cd': 0.004e-4, 'in': 0.001e-4, 'sn': 0.076e-4, 'sb': 0.001e-4,
              'cs': 0.001e-4, 'ba': 0.047e-4, 'la': 0.018e-4, 'eu': 0.001e-4, 'tb': 0.001e-4,
              'yb': 0.001e-4, 'hf': 0.001e-4, 'ta': 0.001e-4, 'w': 0.032e-4, 'ir': 0.001e-4,
              'pt': 0.542e-4, 'hg': 0.001e-4, 'th': 0.001e-4, 'u': 0.001e-4}, density=3.217, metadata={"name":"SiC"}),
    # Nb3Sn superconductor
    Material({'nb': 70.1, 'sn': 29.9}, density=8.9, metadata={"name":"Nb3Sn"}),
    # Modified F82H Ferritic Steel (reference steel for ARIES)
    # R. Klueh et al., Impurity effects on reduced-activation ferritic steels developed for fusion applications, 
    # Journal of Nuclear Materials 280 (2000) 353-359.
    Material({'c':   0.1,     'al':   14e-4,   'v':   0.2,     'cr':   7.5,
              'fe':   90.11586, 'co':   28e-4,   'ni':   474e-4,  'cu':   100e-4,  'nb':
              3.3e-4,  'mo':   21e-4,   'pd':   0.05e-4, 'ag':   0.1e-4,  'cd':   0.4e-4,
              'ta':   0.02,    'w':   2.0,     'os':   0.05e-4, 'ir':   0.05e-4, 'bi':
              0.2e-4,  'eu':   0.05e-4, 'tb':   0.02e-4, 'dy':   0.05e-4, 'ho':   0.05e-4,
              'er':   0.05e-4, 'u':   0.05e-4}, density=7.89, metadata={"name":"MF82H"}),
    # ODS LiPb-corrosion-resistant steel with ''Present'' impurities.
    # Fusion Reactor Materials Program Semi-annual Report for Dec. 2014, DOE/ER- 0313/57  Section 2.1
    # Impurities: R. Klueh et al., Impurity effects on reduced- activation ferritic steels developed for fusion applications, 
    # Journal of Nuclear  Materials 280 (2000) 353-359.
    Material({'c':0.0380, 'n':0.0455, 'o':0.08428, 'al':4.8, 'si':0.02, 's':0.0020,
              'ti':0.01, 'cr':11.4, 'fe':83.356601, 'y':0.19, 'w':0.05, 'co':8e-4,
              'ni':13e-4, 'cu':10e-4, 'nb':0.5e-4, 'mo':5e-4, 'pd':0.05e-4, 'ag':0.05e-4,
              'cd':0.05e-4, 'eu':0.02e-4, 'tb':0.02e-4, 'dy':0.05e-4, 'ho':0.05e-4,
              'er':0.05e-4, 'os':0.05e-4, 'ir':0.05e-4, 'bi':0.05e-4}, density=7.779,
              metadata={"name":"ODS_Alloy_125Y_Present"})]
with open("materials.txt", 'w') as f:
    s = ""
    for i, mat in enumerate(mats):
        s += "{}: {}\n".format(i, mat.metadata["name"])
    f.write(s)

# first_wall, blanket, shield
m = Mesh(structured=True, mesh="fw_n.h5m")
m.flux = IMeshTag(217, float)
flux = m.flux[:]
spectra = [flux[1023, 42:], flux[1767, 42:], flux[2728, 42:]]

# irradiation and decay times in seconds
irr_bounds = np.logspace(0,9,31)
decay_bounds = np.logspace(0,9,31)

names = ["FW", "Shield", "VV"]
mags = [np.sum(spectra[0]), np.sum(spectra[1]), np.sum(spectra[2])]

for i in range(3):
    eta, maxes_sorted, photon_spectra = eta_sweep(mats, irr_bounds, decay_bounds, spectra[i], mags[i])
    with open("{0}.txt".format(names[i]), 'w') as f:
        f.write(maxes_sorted)
    eta.dump("{0}.npy".format(names[i]))
    photon_spectra.dump("{0}_photon_spectra.npy".format(names[i]))
    plot_eta_sweep(eta, mats, irr_bounds, decay_bounds, mags[i], names[i], title=False)

